# base-sender

Базовый сендер для всех наших http-клиентов

# Установка

`composer require iwaydev/base-sender`

# Использование

Пример использования сендера в laravel.
```
$sender = new Sender(
    config('senders.service_name.connection.url'),
    config('senders.service_name.connection.api_version')
);
$response = $sender->get('v1/action');
```
## Кэширование

По дефолту кэширование не используется.

Чтобы установить кэшер нужно передать его инстнс в конструктор.
Время жизни кэша по дефолту равно 3600 сек., меняется также - передачей значения в конструктор.
```
$sender = new Sender(
    config('senders.service_name.connection.url'),
    config('senders.service_name.connection.api_version'),
    cache()->store(),
    config('senders.service_name.cache.lifetime')
);
```
Чтобы использовать инициализированный кэшер нужно в цепочке вызова указать метод `caching()`,
если данный метод не указан, то запрос будет отправлен без кэширования.
```
$response = $sender->caching()->get('v1/action');
```
## Логирование

По дефолту логирование не ведется.

Чтобы начать логирование нужно передать инстанс логгера в конструктор.

```
$sender = new Sender(
    config('senders.service_name.connection.url'),
    config('senders.service_name.connection.api_version'),
    cache()->store(),
    config('senders.service_name.cache.lifetime'),
    logs('service_name')
);
```

В лог пишутся все данные запроса и контент из ответа.

```
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Method: POST  
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Url: http://api.service-name.local/v1/action  
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Options: {"form_params":[],"headers":{"X-UUID":"your-x-uuid","Authorization":"Bearer json.web.token"}} 
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Response. Content: {"data":1} 
```

Если требуется изменить место или способ логирования нужно передать инстанс другого логгера в метод `setLogger()`, добавив его в цепочку вызовов.

**Внимание!** Логгер, переданный в `setLogger()` будет заменен на дефолтный(тот который был передан в конструкторе) после отправки запроса.

```
$response = $sender->get('v1/action'); // Логирование в 'service_name'
$response = $sender->setLogger(logs('logger_a'))->get('v1/action'); // Логирование в 'logger_a'
$response = $sender->setLogger(logs('logger_b'))->get('v1/action'); // Логирование в 'logger_b'
$response = $sender->get('v1/action'); // Логирование снова в 'service_name'
```

## Установка заголовка X-UUID

По дефолту заголовок генерируется при создании сендера.

Если нужно использовать конкретный, то его передаем в конструктор.

```
$sender = new Sender(
    config('senders.service_name.connection.url'),
    config('senders.service_name.connection.api_version'),
    cache()->store(),
    config('senders.service_name.cache.lifetime'),
    logs('service_name'),
    $app->request->header('X-UUID', null)
);
$response = $sender->get('v1/action');
```

Можно менять по ходу использования вызвав метод `setXUuid()`.
Если метод вызван без передачи значения, то будет сгенерирован новый случайный заголовок.

```
$sender->setXUuid(); // Генерирует новый случайный заголовок X-UUID
$sender->setXUuid('хочу-это-заголовок'); // Устанавливает переданное значение как новый заголовок X-UUID
```

## Обработка ошибок от сервиса

По дефолту все исключения в ответе от сервиса, к которому был отправлен запрос, глушатся и возвращаются в ответе метода.

Если нужно кинуть исключение при ошибке на запрос, то в цепочке вызова нужно указать метод `withErrors()`.

```
$response = $sender->get('v1/action'); // В случае ошибки будет возвращен ответ от сервиса
$response = $sender->withErrors()->get('v1/action'); // В случае ошибки будет брошено исключение с текстом ошибки из ответа сервиса
```
